#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::Time;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw( @months @days
			gmtime2 tz_time gmt_date local_date
			time_ago time_rel parse_time split_time
			)
}

our @months = ( 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' );
our @days = ( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' );

sub _time_text($) {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(shift);
	return $mday.'/'.$months[$mon].'/'. substr($year, -2, 2).' '.
		sprintf("%02d:%02d", $hour, $min);
}

sub gmtime2(;$) {
	my ($time) = @_;
	$time = time() unless $time;
	return _time_text($time) . ' GMT';
}

sub tz_time($;$) {
	my ($tzoffset, $time) = @_;
	return _time_text(($time ? $time : time()) + tz_to_offset($tzoffset));
}

sub tz_to_offset($) {
	my ($offset) = @_;
	# offset is a signed integer corresponding to 1/4 hr increments
	# or 900 seconds (15 minutes)
	return ($offset * 900); 
}

sub _date_text($) {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(shift);
	return (!wantarray ? ($year+1900).' '.$months[$mon].' '.$mday : ($year + 1900, $mon+1, $months[$mon], $mday));
}

sub gmt_date(;$) {
	my ($time) = @_;
	$time = time() unless $time;
	return _date_text($time);
}

sub local_date($;$) {
	my ($tzoffset, $time) = @_;
	return _date_text(($time ? $time : time()) + tz_to_offset($tzoffset));
}

sub parse_time($) {
	my ($str) = @_;
	my $out;
	$str =~ s/^\+//;
	$str = lc($str);

	my @vals = split(/(?<!\d)(?=\d+\w)/, $str);

	foreach my $val (@vals) {
		$val =~ /(\d+)(\w)/;
		my ($num, $pos) = ($1, $2);

		if($pos eq 'w') { $num *= (86400*7) }
		elsif($pos eq 'd') { $num *= 86400 }
		elsif($pos eq 'h') { $num *= 3600 }
		elsif($pos eq 'm') { $num *= 60 }
		elsif($pos ne 's') { return undef }

		$out += $num;
	}

	return $out;
}

sub split_time($) {
	my ($difference) = @_;
	my ($weeks, $days, $hours, $minutes, $seconds);
	$seconds 	=  $difference % 60;
	$difference 	= ($difference - $seconds) / 60;
	$minutes    	=  $difference % 60;
	$difference 	= ($difference - $minutes) / 60;
	$hours 		=  $difference % 24;
	$difference 	= ($difference - $hours)   / 24;
	$days 		=  $difference % 7;
	$weeks		= ($difference - $days)    /  7;

	return ($weeks, $days, $hours, $minutes, $seconds);
}

sub time_ago($) {
	return time_rel(time() - $_[0]);
}

sub time_rel($) {
	my ($time) = @_;

	my ($weeks, $days, $hours, $minutes, $seconds) = split_time($time);

	if($time >= 604800) {
		return "$weeks week".
			($weeks!=1 ? 's' : '').
			", $days day".
			($days!=1 ? 's' : '');
	}
	elsif($time >= 86400) {
		return "$days day".
			($days!=1 ? 's' : '').
			", $hours hour".
			($hours!=1 ? 's' : '');
	}
	elsif($time >= 3600) {
		return "$hours hour".
			($hours!=1 ? 's' : '').
			", $minutes minute".
			($minutes!=1 ? 's' : '');
	}
	elsif($time >= 60) {
		return "$minutes minute".
		($minutes!=1 ? 's' : '').
		", $seconds second".
		($seconds!=1 ? 's' : '');
	}
	else {
		return "$seconds second".
		($seconds!=1 ? 's' : '');
	}
}

1;
